import {SyntheticEvent, useCallback, useContext, useMemo, useRef} from 'react';
import {Internals} from 'remotion';
import {PlayerEventEmitterContext} from './emitter-context';
import {PlayerEmitter} from './event-emitter';

export const usePlayer = (): {
	frameBack: (frames: number) => void;
	frameForward: (frames: number) => void;
	isLastFrame: boolean;
	emitter: PlayerEmitter;
	playing: boolean;
	play: (e?: SyntheticEvent) => void;
	pause: () => void;
	seek: (newFrame: number) => void;
	getCurrentFrame: () => number;
	isPlaying: () => boolean;
} => {
	const [playing, setPlaying] = Internals.Timeline.usePlayingState();
	const imperativePlaying = useRef(false);
	const frame = Internals.Timeline.useTimelinePosition();
	const setFrame = Internals.Timeline.useTimelineSetFrame();
	const setTimelinePosition = Internals.Timeline.useTimelineSetFrame();
	const audioContext = useContext(Internals.SharedAudioContext);

	const frameRef = useRef<number>();
	frameRef.current = frame;
	const video = Internals.useVideo();
	const config = Internals.useUnsafeVideoConfig();
	const emitter = useContext(PlayerEventEmitterContext);

	const lastFrame = (config?.durationInFrames ?? 1) - 1;
	const isLastFrame = frame === lastFrame;

	if (!emitter) {
		throw new TypeError('Expected Player event emitter context');
	}

	const seek = useCallback(
		(newFrame: number) => {
			setTimelinePosition(newFrame);
			emitter.dispatchSeek(newFrame);
		},
		[emitter, setTimelinePosition]
	);

	const play = useCallback(
		(e?: SyntheticEvent) => {
			if (imperativePlaying.current) {
				return;
			}

			if (isLastFrame) {
				seek(0);
			}

			if (audioContext && audioContext.numberOfAudioTags > 0 && e) {
				audioContext.playAllAudios();
			}

			imperativePlaying.current = true;
			setPlaying(true);
			emitter.dispatchPlay();
		},
		[isLastFrame, audioContext, setPlaying, emitter, seek]
	);

	const pause = useCallback(() => {
		if (imperativePlaying.current) {
			imperativePlaying.current = false;

			setPlaying(false);
			emitter.dispatchPause();
		}
	}, [emitter, setPlaying]);

	const frameBack = useCallback(
		(frames: number) => {
			if (!video) {
				return null;
			}

			if (imperativePlaying.current) {
				return;
			}

			if (frame === 0) {
				return;
			}

			setFrame((f) => Math.max(0, f - frames));
		},
		[frame, setFrame, video]
	);

	const frameForward = useCallback(
		(frames: number) => {
			if (!video) {
				return null;
			}

			if (imperativePlaying.current) {
				return;
			}

			if (isLastFrame) {
				return;
			}

			setFrame((f) => Math.min(lastFrame, f + frames));
		},
		[isLastFrame, lastFrame, setFrame, video]
	);

	const returnValue = useMemo(() => {
		return {
			frameBack,
			frameForward,
			isLastFrame,
			emitter,
			playing,
			play,
			pause,
			seek,
			getCurrentFrame: () => frameRef.current as number,
			isPlaying: () => imperativePlaying.current as boolean,
		};
	}, [
		emitter,
		frameBack,
		frameForward,
		isLastFrame,
		pause,
		play,
		playing,
		seek,
	]);

	return returnValue;
};
